/********************************************************************************
 Author : Sijo (cic.asia@analog.com)

 Date : December 2013

 File name : ADuCM360Driver.c

 Description : Microcontroller Initilization

 Hardware plateform : ADuCM360 	
********************************************************************************/

#include <ADuCM360.h>
#include "ADuCM360Driver.h"
#include "AD5764.h"
#include "ClkLib.h"
#include "IntLib.h"
#include "DioLib.h"
#include "WdtLib.h"
#include "DioLib.h"

//Delay Routine
void Delay(unsigned long int DelayTime)			  //delay function,Delay(1)=10us
{ 
	unsigned char i;

	while(DelayTime>0)
	{ 
		for(i=0;i<32;i++) {}
		DelayTime--;
	}
	
}

//Initialize ADuC360 for AD5764

void ADuCM360Initialize(void)
{
	
	WdtCfg(T3CON_PRE_DIV1,T3CON_IRQ_EN,T3CON_PD_DIS);  // Disable Watchdog timer resets
                       
	DioOen(pADI_GP1,0xF7);// Set P1.0,P1.1,P1.2 as an output as Sync,Sclk,Sdin
	
	DioPul(pADI_GP1,0x08);// Set weak pull up for P1.3 input SDOUT
	
  //Disable clock to unused peripherals
  ClkDis(CLKDIS_DISSPI0CLK|CLKDIS_DISSPI1CLK|CLKDIS_DISI2CCLK|CLKDIS_DISUARTCLK|CLKDIS_DISPWMCLK|CLKDIS_DIST0CLK|CLKDIS_DIST1CLK|CLKDIS_DISDACCLK|CLKDIS_DISDMACLK|CLKDIS_DISADCCLK);
  ClkCfg(CLK_CD0,CLK_HF,CLKSYSDIV_DIV2EN_DIS,CLK_UCLKCG);            // Select CD0 for CPU clock - 16Mhz clock
	
}

/**
	@brief void SPIinitialize()
	- Generic SPI initialize the SPI pins with default levels
		@ no return value
**/
void SPIinitialize()
{
		//Initialize the serial interface
	ADuCM360GpioOutput(SYNC, PIN_HIGH);
	ADuCM360GpioOutput(SCLK, PIN_LOW);
	ADuCM360GpioOutput(SDIN, PIN_LOW);
}


/**
	@brief void ADuCM360GpioOutput  (unsigned char  GpioNo, unsigned char OutputValue)
	- Configures the GPIO's as output based on pin assigned to the Controllers
		@param GpioNo : Pin number XY, X - Port number, Y - Pin Number
		@ - no return value
**/
void ADuCM360GpioOutput  (unsigned char  GpioNo, unsigned char OutputValue)
{
	unsigned char PortNo  =  GpioNo  & 0xF0; // Extracting the Port number
	unsigned char PinNo  =  GpioNo  & 0x0F;	//Extracting the Pin number
	switch(PortNo)
	{	
		case 0x00:
				if(OutputValue == 0x1)
					pADI_GP0->GPSET =  (1<< PinNo);
				else if(OutputValue == 0x0)
					pADI_GP0->GPCLR =  (1<< PinNo);
				break;
				
		case 0x10:
				if(OutputValue == 0x1)
					pADI_GP1->GPSET =  (1<< PinNo);
				else if(OutputValue == 0x0)
					pADI_GP1->GPCLR =  (1<< PinNo);
				break;
				
		case 0x20: 
				if(OutputValue == 0x1)
					pADI_GP2->GPSET =  (1<< PinNo);
				else if(OutputValue == 0x0)
					pADI_GP2->GPCLR =  (1<< PinNo);
				break;
	}
	
}

/**
	@brief unsigned int ADuCM360GpioInput (unsigned char  GpioNo)
	- Configures the GPIO's as input based on pin assigned to the Controllers
		@param GpioNo : Pin number XY, X - Port number, Y - Pin Number
		@return Status
		Status - returns either high or low
**/
unsigned int ADuCM360GpioInput (unsigned char  GpioNo)
{
	unsigned char PortNo  =  GpioNo  & 0xF0;
	unsigned char PinNo  =  GpioNo  & 0x0F;
	unsigned char Status=0;
	switch(PortNo)
	{	
		case 0x00:
				if((1<< PinNo) == ((pADI_GP0->GPIN) & (1<< PinNo)))     // Returns the value at Dout pin bit by bit.
					Status = 1;
				else
					Status =0;
				break;
				
		case 0x10:
				if((1<< PinNo) == ((pADI_GP1->GPIN) & (1<< PinNo)))     // Returns the value at Dout pin bit by bit.
					Status = 1;
				else
					Status =0;
				break;
				
		case 0x20: 
				if((1<< PinNo) == ((pADI_GP2->GPIN) & (1<< PinNo)))     // Returns the value at Dout pin bit by bit.
					Status = 1;
				else
					Status =0;
				break;
				
		default:
				break;
	}
	
	return(Status);	
}

/**
	@brief void SpiReadWrite(unsigned char* InputBuffer, unsigned char* OutputBuffer, unsigned char NoOfByte)
	- Generic SPI Read and Write
		@param InputBuffer :8 bit output data array
           OutputBuffer: 8 bit input data array
		NoOfBytes : (0x00-0x20) Number of Bytes required to transfer
		@ no return value
**/
void SpiReadWrite(unsigned char* InputBuffer, unsigned char* OutputBuffer, unsigned char NoOfByte)
{
	unsigned char ReceivedByte=0x00;
	unsigned char Bit, Byte;
	unsigned char Register;
	unsigned char ReceivedBit = 0x00;
		
	ADuCM360GpioOutput(SYNC, PIN_LOW);//Start Reading
	
	for(Byte=0; Byte< NoOfByte; Byte++)
  {
		
		Register = *OutputBuffer; //Moving the Higer Byte to Register for Transfer
				
		for(Bit=0; Bit<8; Bit++)
		{
			ADuCM360GpioOutput(SCLK, PIN_HIGH);
			if(0x80==(Register & 0x80))
			{
				ADuCM360GpioOutput(SDIN, PIN_HIGH);
			}
			else
			{
				ADuCM360GpioOutput(SDIN, PIN_LOW);
			}
			Delay(1);			
			ADuCM360GpioOutput(SCLK, PIN_LOW); //Input data latched at falling edge of clock
			Delay(1);
			Register <<= 1;  //Rotate data
			ReceivedByte <<= 1;
			ReceivedBit = ADuCM360GpioInput(SDOUT); //Data read from SDOUT pin
			if(ReceivedBit>0x0000) //Received bit is high
			{
				ReceivedByte |=1;
			}
			Delay(1);
		}
		
		*InputBuffer= ReceivedByte; //Writing the Received Byte to the Input Buffer
		InputBuffer++;
		OutputBuffer++;
		ReceivedByte = 0x00; //Reinitialize the Registers
  }
	Delay(1);
	ADuCM360GpioOutput(SYNC, PIN_HIGH); //Write over
}


